#include <io.h>
#include <dos.h>
#include <fcntl.h>
#include <conio.h>
#include <alloc.h>
#include <stdio.h>
#include <share.h>
#include <string.h>
#include <sys\stat.h>
#include <process.h>
#include <strstream.h>
#include "msgcls.h"
#include "bsreal.h"

//******************[ MISC ROUTINES USED BY MSG ROUTINES ]***************

void exit_err(int error_code){
        switch(error_code){
                case LOCK_ERROR:
                        cout << CR"Error encountered during a file lock"CR;
                        break;
                case WRHDR_ERROR:
                        cout << CR"Error encountered during header write"CR;
						break;
				case SEEK_ERROR:
						cout << CR"Error attempting seeking message in main message base";
						break;
				case READ_ERROR:
						cout << CR"Error encountered trying to read in message header";
						break;
				case MEM_ERROR:
						cout << CR"Error encountered attempting to allocate memory";
						break;
		}
        exit(error_code);
}

//*******************************[ MSG CLASS ]****************************
//*******************************[ MSG CLASS ]****************************
//*******************************[ MSG CLASS ]****************************


MBH::MBH(char *fn,int nwf){    // CONSTRUCTOR opens and reads in header data
		int fnsize=strlen(fn);
		network_flg=nwf;       // Multi-node flag;
        memmove(filename,fn,fnsize+1);

        if((fhandle=open(filename,O_RDONLY|O_BINARY|O_DENYNONE))==-1){
                cout << CR"Attempted to open " << filename << CR;
                perror(CR"Error encountered reading message base header:");
                exit(1);}

                if((read(fhandle,&MBHdata,sizeof(MsgBaseHdr)))==-1){
                cout << CR"Attempted to read " << filename << CR;
                cout << CR"Read failed in MBH constructor (message base header)"CR;
                exit(1);}

}

MBH::~MBH(){            // DESTRUCTOR
                close(fhandle);
}

//***************************[ GET DATA ]**********************************

int MBH::get_network_flg(){             //  NETWORK FLAG IS SET BY CONSTRUCTOR (YOU)
        return(network_flg);}

ulong MBH::get_high_msg(){              //  GET HIGH MESSAGE
                BSreal ghm(MBHdata.high_msg);
                return(ghm.get_long_value());}

ulong MBH::get_low_msg(){               //      GET LOW MESSAGE
                BSreal glm(MBHdata.low_msg);
                return(glm.get_long_value());}

ulong MBH::get_act_msgs(){              //      GET ACTIVE MESSAGES
                BSreal gam(MBHdata.act_msgs);
                return(gam.get_long_value());}

ulong MBH::get_sys_callers(){   //      GET SYSTEM CALLERS (number)
                BSreal gsc(MBHdata.sys_callers);
                return(gsc.get_long_value());}

int MBH::get_locked(){                  //  GET LOCKED STATUS
        int status;
        if (network_flg==YES)
                if ((status=lock(fhandle,16,6))==0){
                        unlock(fhandle,16,6);
                        return(NO);}
                else    return(YES);
        return(NA);}

int MBH::wait_get_locked(int seconds){
        int status;
        while((status=get_locked())==YES){
                cout << CR << filename << " is busy.  Will keep checking for " << seconds << " seconds.";
                sleep(1);
                seconds--;
                if(seconds<1) break;}
        return(status);}


//********************[ SET DATA USING DOUBLE (or less) ]***************

ulong MBH::set_high_msg(double Value){  // SET HIGH MESSAGE NUMBER
        BSreal shm(Value);
                shm.copy_BSreal_to(MBHdata.high_msg);
        return(shm.get_long_value());}

ulong MBH::set_low_msg(double Value){   // SET LOW MESSAGE NUMBER
        BSreal slm(Value);
                slm.copy_BSreal_to(MBHdata.low_msg);
        return(slm.get_long_value());}

ulong MBH::set_act_msgs(double Value){  // SET ACTIVE MESSAGES
        BSreal sam(Value);
                sam.copy_BSreal_to(MBHdata.act_msgs);
        return(sam.get_long_value());}

ulong MBH::set_sys_callers(double Value){       // SET SYSTEM CALLERS
        BSreal ssc(Value);
                ssc.copy_BSreal_to(MBHdata.sys_callers);
        return(ssc.get_long_value());}

int MBH::set_lock(){                            //  SET LOCKED STATUS
        if (network_flg==YES)
                if((lock(fhandle,16,6))==0)
                        return(YES);
        else    return(NO);
        return(NA);}

int MBH::remove_lock(){
        if (network_flg==YES)
                if ((unlock(fhandle,16,6))==0)
                                return(YES);
                else    return(NO);
                return(NA);}

int MBH::wait_set_lock(int seconds){
        int status;
        while(((status=set_lock())==NO) && (seconds>0)){
                cout << CR << filename << " is busy.  Will keep trying for " << seconds << " seconds.";
                sleep(1);
                seconds--;}
        return(status);}


//******************[ SET DATA USING BSREAL ]***************************

ulong MBH::set_high_msg(bsreal Value){  // SET HIGH MESSAGE NUMBER
        BSreal shm(Value);
		shm.copy_BSreal_to(MBHdata.high_msg);
		return(shm.get_long_value());}

ulong MBH::set_low_msg(bsreal Value){   // SET LOW MESSAGE NUMBER
        BSreal slm(Value);
		slm.copy_BSreal_to(MBHdata.low_msg);
		return(slm.get_long_value());}

ulong MBH::set_act_msgs(bsreal Value){  // SET ACTIVE MESSAGES
        BSreal sam(Value);
		sam.copy_BSreal_to(MBHdata.act_msgs);
		return(sam.get_long_value());}

ulong MBH::set_sys_callers(bsreal Value){// SET SYSTEM CALLERS
        BSreal ssc(Value);
		ssc.copy_BSreal_to(MBHdata.sys_callers);
		return(ssc.get_long_value());}


//*******************[ WRITE MESSAGE BASE HEADER TO DISK ]*****************

void MBH::save_mbh(int seconds){
        whandle = sopen(filename, O_WRONLY, SH_DENYNO, S_IWRITE);
		MBH_flg=YES;
        if (!whandle){
                cout << CR"Error encountered trying to open " << filename;
                exit_err(WRHDR_ERROR);}

        if (network_flg==YES){          // We gotta wait if other locks are on.
                if ((wait_get_locked(seconds))==YES) // if still locked after
                        exit_err(LOCK_ERROR);}                   // seconds is up; adios...

                if((write(whandle,&MBHdata,sizeof(MsgBaseHdr)))==-1){
                cout << CR"Error encountered trying to write header to "<< filename;
                exit_err(WRHDR_ERROR);}

        if (network_flg==YES){          // Set locks after open/write header.
                if((wait_set_lock(seconds))==NOGOOD)// Not sure if this second
                        exit_err(LOCK_ERROR);}                  // check is even required.
}

//*********************[ CLOSE AND UNLOCK MESSAGE FILE ]*****************

void MBH::close_mbh(){
        close(whandle);
        remove_lock();
}

//***********************************************************************
